/**
 * Created by AlexanderC on 6/10/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LocalRequest = undefined;

var _SuperagentResponse = require('./SuperagentResponse');

var _LambdaResponse = require('./LambdaResponse');

var _Request = require('./Request');

var _Action = require('./Action');

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _MissingLocalLambdaExecWrapperException = require('./Exception/MissingLocalLambdaExecWrapperException');

var _MissingLambdaLocalPathException = require('./Exception/MissingLambdaLocalPathException');

var _urlParse = require('url-parse');

var _urlParse2 = _interopRequireDefault(_urlParse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Resource request instance
 */
class LocalRequest extends _Request.Request {
  constructor(...args) {
    super(...args);
  }

  /**
   * @returns {String}
   * @private
   */
  _buildEndpointUrl() {
    let endpoint = this._async ? LocalRequest.LOCAL_LAMBDA_ASYNC_ENDPOINT : LocalRequest.LOCAL_LAMBDA_ENDPOINT;

    if (this.baseUrl) {
      let urlParts = (0, _urlParse2.default)(this.baseUrl);
      let protocol = urlParts.protocol || 'http:';
      let baseUrl = `${protocol}//${urlParts.host}`;

      endpoint = baseUrl + endpoint;
    }

    return endpoint;
  }

  /**
   * @param {Function} callback
   * @returns {LocalRequest|*}
   */
  _send(callback = () => {}) {
    let actionType = this._action.type;
    let securityService = this._action.resource.security;

    if (actionType === _Action.Action.LAMBDA) {
      let data = {
        lambda: this._action.source.original,
        payload: this.payload,
        method: this._method,
        context: {
          identity: {
            cognitoIdentityPoolId: securityService.token ? securityService.token.identityPoolId : null,
            cognitoIdentityId: securityService.token ? securityService.token.identityId : null,
            isAnonymous: securityService.token ? securityService.token.isAnonymous : true
          }
        }
      };

      if (typeof window === 'undefined') {
        if (!global.hasOwnProperty(LocalRequest.LAMBDA_EXEC_WRAPPER_KEY)) {
          throw new _MissingLocalLambdaExecWrapperException.MissingLocalLambdaExecWrapperException(LocalRequest.LAMBDA_EXEC_WRAPPER_KEY);
        }

        let localPath = this._action.source._localPath;

        if (!localPath) {
          throw new _MissingLambdaLocalPathException.MissingLambdaLocalPathException(data.lambda);
        }

        let execWrapper = global[LocalRequest.LAMBDA_EXEC_WRAPPER_KEY];

        execWrapper[this._async ? 'invokeAsync' : 'invoke'](localPath, data, (error, result) => {
          let resultData = {};

          if (this._async) {
            resultData.Status = 202;
          } else {
            resultData.Payload = result;
          }

          callback(new _LambdaResponse.LambdaResponse(this, error ? null : resultData, error));
        });
      } else {
        _superagent2.default.post(this._buildEndpointUrl()).send(data).end((error, response) => {
          callback(new _SuperagentResponse.SuperagentResponse(this, response, error));
        });
      }
    } else {
      return this.constructor.useNative()._send(...arguments);
    }

    return this;
  }

  /**
   * @returns {String}
   */
  static get LOCAL_LAMBDA_ENDPOINT() {
    return '/_/lambda';
  }

  /**
   * @returns {String}
   */
  static get LOCAL_LAMBDA_ASYNC_ENDPOINT() {
    return '/_/lambda-async';
  }

  /**
   * @returns {String}
   */
  static get LAMBDA_EXEC_WRAPPER_KEY() {
    return '_deep_lambda_exec_';
  }
}
exports.LocalRequest = LocalRequest;