/**
 * Created by CCristi on 11/14/16.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StrategyFactory = undefined;

var _CustomStrategy = require('./CustomStrategy');

var _deepCore = require('deep-core');

var _deepCore2 = _interopRequireDefault(_deepCore);

require('./InternalErrorStrategy');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// browserify fix

class StrategyFactory {
  /**
   * @param {String|Function} strategy
   * @param {Object[]} args
   * @returns {AbstractStrategy}
   */
  static create(strategy = 'internal-error', ...args) {
    if (typeof strategy === 'string') {
      const strategyFullName = `${StrategyFactory._pascalCase(strategy)}Strategy`;
      const StrategyProto = require(`./${strategyFullName}`)[strategyFullName];

      return new StrategyProto(...args);
    } else if (typeof strategy === 'function') {
      return new _CustomStrategy.CustomStrategy(strategy, ...args);
    }

    throw new _deepCore2.default.Exception.InvalidArgumentException(strategy, 'string|function');
  }

  /**
   * @param {String} str
   * @returns {String}
   */
  static _pascalCase(str) {
    return str.split(/[^a-zA-Z0-9]+/).reduce((pascalString, part) => {
      return pascalString + part.charAt(0).toUpperCase() + part.slice(1);
    }, '');
  }
}
exports.StrategyFactory = StrategyFactory;